/*
 * Copyright 2007 by Kappich Systemberatung, Aachen
 * Copyright 2005 by Kappich+Kni Systemberatung Aachen (K2S)
 * 
 * This file is part of de.bsvrz.dav.daf.
 * 
 * de.bsvrz.dav.daf is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.dav.daf is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with de.bsvrz.dav.daf; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.dav.daf.main.config;

import java.util.*;

/**
 * Diese Schnittstelle definiert Methoden, um auf Elemente von Zusammenstellungen von System-Objekten zugreifen zu knnen. Sie wird benutzt bei <ul> <li>{@link
 * ObjectSet Mengen}, um auf die Elemente einer Menge zugreifen zu knnen;</li> <li>{@link SystemObjectType Typen}, um auf alle Objekte des jeweiligen Typs
 * zugreifen zu knnen;</li> </ul>
 *
 * @author Kappich Systemberatung
 * @version $Revision: 5052 $
 */
public interface SystemObjectCollection {

	/**
	 * Bestimmt die zum aktuellen Zeitpunkt zur Zusammenstellung gehrenden Elemente.
	 *
	 * @return Liste mit den aktuell zur Zusammenstellung gehrenden System-Objekten.
	 */
	public List<SystemObject> getElements();

	/**
	 * Bestimmt die zu einem vorgegebenen Zeitpunkt zur Zusammenstellung gehrenden Elemente. Nicht dynamische Mengen mit Referenzierungsart Komposition oder
	 * Aggregation liefern immer alle Elemente der Menge zurck.
	 *
	 * @param time Zeitpunkt in Millisekunden seit 1970
	 *
	 * @return Liste mit den zum angegebenen Zeitpunkt zur Zusammenstellung gehrenden System-Objekten.
	 */
	public List<SystemObject> getElements(long time);

	/**
	 * Bestimmt die Elemente, die an mindestens einem Zeitpunkt des angegebenen Zeitbereichs zur Zusammenstellung gehrt haben. Nicht dynamische Mengen mit
	 * Referenzierungsart Komposition oder Aggregation liefern immer alle Elemente der Menge zurck.
	 *
	 * @param startTime Erster Zeitpunkt des Zeitbereichs in Millisekunden seit 1970.
	 * @param endTime   Letzter Zeitpunkt des Zeitbereichs in Millisekunden seit 1970.
	 *
	 * @return Liste mit den zu mindestens einem Zeitpunkt des Zeitbereichs zur Zusammenstellung gehrenden System-Objekten.
	 */
	public List<SystemObject> getElementsInPeriod(long startTime, long endTime);

	/**
	 * Bestimmt die Elemente, die whrend des gesamten angegebenen Zeitbereichs zur Zusammenstellung gehrt haben. Nicht dynamische Mengen mit Referenzierungsart
	 * Komposition oder Aggregation liefern immer alle Elemente der Menge zurck.
	 *
	 * @param startTime Erster Zeitpunkt des Zeitbereichs in Millisekunden seit 1970.
	 * @param endTime   Letzter Zeitpunkt des Zeitbereichs in Millisekunden seit 1970.
	 *
	 * @return Liste mit den whrend des gesamten Zeitbereichs zur Zusammenstellung gehrenden System-Objekten.
	 */
	public List<SystemObject> getElementsDuringPeriod(long startTime, long endTime);
}

