/*
 * Copyright 2007 by Kappich Systemberatung Aachen 
 * 
 * This file is part of de.bsvrz.puk.config.
 * 
 * de.bsvrz.puk.config is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * de.bsvrz.puk.config is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with de.bsvrz.puk.config; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

package de.bsvrz.puk.config.configFile.datamodel;

import de.bsvrz.dav.daf.main.config.ConfigurationArea;
import de.bsvrz.dav.daf.main.config.management.ConfigAreaAndVersion;

import java.util.*;

/**
 * Die Aufgabe der Implementation dieses Interfaces ist es, fehlende Konfigurationsbereiche, die bentigt werden um Abhngigkeiten zwischen
 * Konfigurationsberichen aufzulsen, zu entdecken.
 * <p/>
 * Jeder Konfigurationsbereich speichert die Konfigurationsbereiche von denen er abhngig ist. Soll der Konfigurationsbereich nun benutzt werden (Import,
 * Export, Aktivierung, Freigabe zur bernahme, usw.), so mssen diese Bereiche vorhanden sein.
 *
 * @author Kappich Systemberatung
 * @version $Revision: 5084 $
 */
public interface AreaDependencyCheck {

	/**
	 * Die Methode prft ob alle Abhngigkeiten der einzelnen Bereiche mit den bergebenen Bereichen aufgelst werden knnen.
	 * <p/>
	 *
	 * @param areas Bereiche mit entsprechenden Vesionen(>0). Alle Abhngigkeiten der Bereiche mssen mit den bergebenen Bereichen aufgelst werden knnen.
	 *
	 * @return Objekt, dass das Ergebnis der Prfung enthlt.
	 */
	public AreaDependencyCheckResult checkAreas(List<ConfigAreaAndVersion> areas);

	public interface AreaDependencyCheckResult {

		/**
		 * Gibt alle Bereiche und deren Abhngigkeiten zurck, die nicht aufgelst werden konnten und deren Abhngigkeit als "optional" gekennzeichnet sind.
		 *
		 * @return Als Schlssel dient der Bereich, dessen Abhngigkeit nicht aufgelst werden konnte. Es wird eine Liste zurckgegeben, mit allen Abhngigkeiten, die
		 *         nicht aufgelst werden konnten und die optional sind. Wurden keine Fehler gefunden, ist die Map leer.
		 */
		public Map<ConfigurationArea, List<ConfigurationAreaDependency>> getOptionalDependencyErrors();

		/**
		 * Gibt alle Bereiche und deren Avhngigkeiten zurck, die nicht aufgelst werden konnten und deren Abhngigkeit als "notwenig" gekennzeichnet sind.
		 *
		 * @return Als Schlssel dient der Bereich, dessen Abhngigkeit nicht aufgelst werden konnte. Es wird eine Liste zurckgegeben, die alle Abhngigkeiten des
		 *         Schlssel-Bereichs enthlt. Wurden keine Fehler gefunden, ist die Map leer.
		 */
		public Map<ConfigurationArea, List<ConfigurationAreaDependency>> getNeededDependencyErrors();

		/**
		 * Gibt alle Bereiche zurck, deren Abhgigkeiten nicht geprft werden konnte. Dies ist zum Beispiel mglich, wenn ein Bereich vor der Einfhrung dieses
		 * Mechanismus aktiviert wurde. Sobald dieser Bereich allerdings aktiviert/zur Aktivierung freigegeben/ zur bernahme freigegeben wird, stehen diese
		 * Informationen zur Verfgung.
		 *
		 * @return Alle Bereiche, deren Abhngigkeiten noch nicht erfasst wurden.
		 */
		public List<ConfigurationArea> getAreasWithUnknownDependencies();
	}

	/** Gibt alle Aktionen an, die die Konfiguration beim Start durchfhren kann. Anhand dieser Aktion werden unterschiedliche Tests durchgefhrt. */
	public enum KindOfConfigurationAction {
		START_CONFIGURATION,
		IMPORT,
		EXPORT,
		ACTIVATION,
		RELEASE_AREAS_FOR_TRANSFER,
		RELEASE_AREAS_FOR_ACTIVATION,;
	}
}
